/*

  Nullsoft WASABI Source File License

  Copyright 1999-2001 Nullsoft, Inc.

    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.


  Brennan Underwood
  brennan@nullsoft.com

*/

// ------------------------------------------------------------------
// ExampleBWndCreate.cpp : Specialized window creation service
// 
//   Provides window creation service for GUIDS
//   Populates component buckets based on Window type
//   Keeps a list of its pointers to delete multiple inheritances if necessary
// ------------------------------------------------------------------

#include "../common/common.h"
#include "../common/bucketitem.h"
#include "resource.h"

#include "ExampleB.h"
#include "ExampleBWnd.h"
#include "ExampleBWndCreate.h"

//
//  This code is slated to be made generic and streamlined.
//
//  However, for now, if you want to make a window, this is how you do it:
//

// NOTE: This code is utilized and linked into the package through the lines
// of code marked as part of the "WindowCreationServices" in ExampleB.cpp.
// Please read the info contained there, first.  You can, almost, copy this
// file and the header for it to make it work blindly in your own project.
//
// You need only edit the #define's in the header, and change the file names
// where appropriate.


// ===========================================================================
//
// Wasabi is asking us if we can create this GUID's window
int WAC_WNDCREATE::testGuid(GUID g) {
  // Test the GUID against our WAC's GUID.  If we match, say so.
  if (g == WACNAME->getGUID()) {
    return 1;
  }
  return 0;
}


// ===========================================================================
//
// Create window according to GUID, if a window class does not support multiple instances,
// you should keep a pointer to it and always return it for its GUID, otherwise you should
// keep a list of your pointers for later deletion
RootWnd *WAC_WNDCREATE::createWindowByGuid(GUID g, RootWnd *parent) {
  // Again test the given GUID against the WAC's GUID.
  if (g == WACNAME->getGUID()) {
    // If we match, create the window.
    return WACNAME->createWindow(0, parent);
  }
  return NULL; 
}


// ===========================================================================
//
// Populate a component bucket with a specific type of window. This function is called
// as long as you do not return NULL, with n incremented for each pass.
//
// NOTE: The only window type you will be called with, for now, is a type of
// "WindowTypes::BUCKETITEM" -- and unless you want to use multiple window types,
// don't worry about returning anything but NULL for values of n greater than 0;
RootWnd *WAC_WNDCREATE::createWindowOfType(int windowtype, int n, RootWnd *parent) {
  switch (windowtype) {
    case WindowTypes::BUCKETITEM: {  
      switch (n) {
        case 0: {
          BucketItem *bucketitem = new BucketItem;
          bucketitem->setAutoOpen(WACNAME->getGUID());
          bucketitem->setAutoText(_("Example B"));
          bucketitem->setBitmaps(WACNAME->gethInstance(), IDB_TAB_NORMAL, NULL, IDB_TAB_HILITED, IDB_TAB_SELECTED);
          bucketitem_list.addItem(bucketitem);
          return bucketitem;
        }
      }
    }
  }
  return NULL;
}


// ===========================================================================
//
//  This callback method is called upon program exit, when the user destroys a window,
// or when the skin is switched.
//
// Walk our lists and destroy our windows cleanly
int WAC_WNDCREATE::destroyWindow(RootWnd *w) {
  // Doublecheck that our bucketitem_list has this window pointer.
  if (bucketitem_list.haveItem(w)) { 
    bucketitem_list.removeItem(w); 
    delete static_cast<BucketItem *>(w); // This is safe, because we already checked it.
  } else {
    // If we don't have it, we can't delete it.  Tell the API.
    return 0;
  }
  return 1;
}

